//----------------------------------------------------------------------------------------------

// NSBezierPathAdditions.m

//----------------------------------------------------------------------------------------------

// Copyright:	© 2014 by Sequoia Aircraft Corporation., all rights reserved.

/*



*/

#import "NSBezierPathAdditions.h"
//#import <complex.h>

@implementation NSBezierPath (Benchmark)

- (void)appendBezierPathWithCrossOfSize:(float)size
{
  float m = size;
  NSPoint pt = [self currentPoint];
  
  [self moveToPoint:NSMakePoint(pt.x - m, pt.y)];
  [self lineToPoint:NSMakePoint(pt.x + m, pt.y)];
  
  [self moveToPoint:NSMakePoint(pt.x, pt.y - m)];
  [self lineToPoint:NSMakePoint(pt.x, pt.y + m)];
  
  [self moveToPoint:pt];
}    

/*
- (void)appendHorizontalSlashOfSize:(float)size
{
  float m = size;
  NSPoint pt = [self currentPoint];
  
  [self moveToPoint:NSMakePoint(pt.x - m,pt.y)];
  [self lineToPoint:NSMakePoint(pt.x + m,pt.y)];
  
  [self moveToPoint:pt];
} 


//float d = MIN(rect.size.height, rect.size.width);
- (void)appendWTRoundedRect:(NSRect)theRect
{
  const double FK = (4.0 / 3.0) * (sqrt(2.0) - 1);
  const double kAdj = 1;  // to adjust the rectangle down to fit text objects better
  
  double TopY    = NSMaxY(theRect) - kAdj;
  double LeftX   = NSMinX(theRect);
  double BottomY = NSMinY(theRect) - kAdj;
  double RightX  = NSMaxX(theRect);

  double TheRad = NSHeight(theRect) / 2;
  double KR = TheRad * FK;
  
  NSPoint A = NSMakePoint(RightX - TheRad     ,TopY              );  // top right
	NSPoint B = NSMakePoint(RightX - TheRad + KR,TopY              );
	NSPoint C = NSMakePoint(RightX              ,TopY - TheRad + KR);
	NSPoint D = NSMakePoint(RightX              ,TopY - TheRad     );
  
  NSPoint E = NSMakePoint(D.x,BottomY + TheRad - KR);
  NSPoint F = NSMakePoint(B.x,BottomY);
  NSPoint G = NSMakePoint(A.x,BottomY);
  
  NSPoint H = NSMakePoint(LeftX + TheRad,BottomY);
  
  NSPoint J = NSMakePoint(LeftX + TheRad - KR,BottomY);
  NSPoint K = NSMakePoint(LeftX,E.y);
  NSPoint L = NSMakePoint(LeftX,D.y);
 
  NSPoint M = NSMakePoint(L.x,C.y);
  NSPoint N = NSMakePoint(J.x,A.y);
  NSPoint O = NSMakePoint(H.x,A.y);

  [self moveToPoint:A];
  [self curveToPoint:D
       controlPoint1:B
       controlPoint2:C];
  
  [self curveToPoint:G
       controlPoint1:E
       controlPoint2:F];  
         
  [self lineToPoint:H];
  
  [self curveToPoint:L
        controlPoint1:J
        controlPoint2:K];
        
  [self curveToPoint:O
        controlPoint1:M
        controlPoint2:N];
  
  [self lineToPoint:A];
};


- (void)appendWideArrow:(NSRect)theRect arrowUp:(BOOL)arrowUp
{
  NSPoint CtrPt = NSMakePoint(NSMidX(theRect),NSMidY(theRect));
	NSPoint A;
	NSPoint B;
	NSPoint C;
	NSPoint D;
	NSPoint DD;
	NSPoint E;
	NSPoint F;
	NSPoint FF;
	NSPoint G;
  
  double HalfWidth = NSWidth(theRect) / 2;
  double HalfHgt   = NSHeight(theRect) / 2;
  double Hgt2Base  = HalfHgt - 8;
  double Overhang  = HalfHgt;
  
	if (arrowUp) {
	  A = NSMakePoint(CtrPt.x - HalfWidth,CtrPt.y - HalfHgt);
	  B = NSMakePoint(CtrPt.x + HalfWidth,A.y);
	  C = NSMakePoint(B.x,CtrPt.y + Hgt2Base);
	  D = NSMakePoint(C.x + Overhang,C.y);
	  DD = NSMakePoint(D.x,D.y + 2);
	  E = NSMakePoint(CtrPt.x,C.y + Overhang);
	  F = NSMakePoint(A.x - Overhang,C.y);
	  FF = NSMakePoint(F.x,DD.y);
	  G = NSMakePoint(A.x,C.y);
	}
	else {
	  A = NSMakePoint(CtrPt.x - HalfWidth,CtrPt.y + HalfHgt);
	  B = NSMakePoint(CtrPt.x + HalfWidth,A.y);
	  C = NSMakePoint(B.x,CtrPt.y - Hgt2Base);
	  D = NSMakePoint(C.x + Overhang,C.y);
	  DD = NSMakePoint(D.x,D.y - 2);
	  E = NSMakePoint(CtrPt.x,C.y - Overhang);
	  F = NSMakePoint(A.x - Overhang,C.y);
	  FF = NSMakePoint(F.x,DD.y);
	  G = NSMakePoint(A.x,C.y);
	};
  
  
  [self moveToPoint:A];
  [self lineToPoint:B];
  [self lineToPoint:C];
  [self lineToPoint:D];
  [self lineToPoint:DD];
  [self lineToPoint:E];
  [self lineToPoint:FF];
  [self lineToPoint:F];
  [self lineToPoint:G];
  [self lineToPoint:A];

};


- (void)appendAirfoil:(double)theLgt
{
  NSPoint A = NSMakePoint(0.0,0.0);
  
  A.x -= (theLgt / 2);
  
  NSPoint B = A;
  B.x += (0.35458 * theLgt);
  B.y += (0.06944 * theLgt);
  
  NSPoint D = A;
  D.x += theLgt;
  
  NSPoint C = A;
  C.x += (0.875 * theLgt);
  C.y += (0.08681 * theLgt);

  NSPoint E = A;
  NSPoint F = A;
  
  E.x += (0.98264 * theLgt);
  E.y += (-0.03125 * theLgt);
  
  F.x += (0.36111 * theLgt);
  F.y += (-0.02778 * theLgt);
  
  [self moveToPoint:A];
  
  [self curveToPoint:D
       controlPoint1:B
       controlPoint2:C];  
         
  [self curveToPoint:A
       controlPoint1:E
       controlPoint2:F];
};
*/


@end
