//----------------------------------------------------------------------------------------------

// OpenClipUtils.m

//----------------------------------------------------------------------------------------------

// Copyright:	© 2014 by Sequoia Aircraft Corporation., all rights reserved.

/*



*/

#import "OpenClipUtils.h"

#import "OpenClipDefines.h"

//----------------------------------------------------------------------------------------------

double Radians2DegreesOpenClip(double Radians)
{
  return(Radians * 180 / pi);
};


double Degrees2RadiansOpenClip(double Degrees)
{
  return(Degrees * pi / 180.0);
};


NSPoint RadialNSPointOpenClip(NSPoint ThePt,double Angle,double Radius)
{
  double TheRadians = Degrees2RadiansOpenClip(Angle);
  double TheSin = sin(TheRadians);
  double TheCos = cos(TheRadians);
  NSPoint NewPt;
  NewPt.x = ThePt.x + (TheCos * Radius);
  NewPt.y = ThePt.y + (TheSin * Radius);
  return NewPt;
};


double Zeroto360OpenClip(double TheAngle)
{
	double	ThisAngle = TheAngle;
  
  if (ThisAngle > 360.0) {
    
    do {
      ThisAngle = ThisAngle - 360.0;
    }
    while (!(ThisAngle <= 360.0));
    
  }
  else if (ThisAngle < 0.0) {
        
    do {
      ThisAngle = ThisAngle + 360.0;
    }
    while (!(ThisAngle >= 0.0));
    
  };
  
  return(ThisAngle);
};



static double RRArcTanOpenClip(NSPoint A,NSPoint B)
{
  double  DX =   B.x - A.x;    // positive DX is to the right
  //double  DY = -(B.y - A.y);   // positive DY is up
  double  DY = B.y - A.y;   // positive DY is up
  double  TheAng;

  if (DX == 0) { // vertical

    if (DY > 0)
    TheAng = 90;
    else
    TheAng = -90;

  }
  else
    TheAng = Radians2DegreesOpenClip(atan(DY/DX));

  if (DX < 0)      // to the left
    TheAng = 180 + TheAng;  //  180 + 45 = 225 or 180 + (-45) = 135

  return(TheAng);
};


double GetAngleOpenClip(NSPoint A,NSPoint B)
{
  return(Zeroto360OpenClip(RRArcTanOpenClip(A,B)));
};


//----------------------------------------------------------------------------------------------

NSMutableDictionary* SetXYPointOpenClip(double X,double Y)
{
  NSNumber *nX = [NSNumber numberWithDouble:X];
  NSNumber *nY = [NSNumber numberWithDouble:Y];
  
  NSMutableDictionary *PtDict	= [NSMutableDictionary dictionary];
  [PtDict setObject:nX forKey:OpenClip_X];
  [PtDict setObject:nY forKey:OpenClip_Y];
  
  return PtDict;
};


NSMutableDictionary* SetXYZPointOpenClip(double X,double Y,double Z)
{
  NSNumber *nX = [NSNumber numberWithDouble:X];  // this is how you add an XYZ point if you want to
  NSNumber *nY = [NSNumber numberWithDouble:Y];
  NSNumber *nZ = [NSNumber numberWithDouble:Z];

  NSMutableDictionary *PtDict	= [NSMutableDictionary dictionary];
  [PtDict setObject:nX forKey:OpenClip_X];
  [PtDict setObject:nY forKey:OpenClip_Y];
  [PtDict setObject:nZ forKey:OpenClip_Z];
  
  return PtDict;
};


//----------------------------------------------------------------------------------------------

BOOL GetScaleOpenClip(NSDictionary *TheDict,double *TheScale)
{
  BOOL Result = NO;
  
  NSNumber *nScale = [TheDict objectForKey:OpenClip_DrawingScale];
  
  if (nScale) {
    *TheScale = [nScale doubleValue];
    Result = YES;
  };
  
  return Result;
};


void SetScaleOpenClip(double TheScale,NSMutableDictionary *TheDict)
{
  NSNumber *nTheScale = [NSNumber numberWithDouble:TheScale];
  [TheDict setObject:nTheScale forKey:OpenClip_DrawingScale];
};



BOOL GetGlobalOriginOpenClip(NSDictionary *TheDict,NSPoint *TheOrigin)
{
  BOOL Result = NO;
  
  NSDictionary *PtDict = [TheDict objectForKey:OpenClip_GlobalOrigin];

  if (PtDict) {
    NSNumber *nX = [PtDict objectForKey:OpenClip_X];
    NSNumber *nY = [PtDict objectForKey:OpenClip_Y];
    
    if (nX && nY) {
      TheOrigin->x = [nX doubleValue];
      TheOrigin->y = [nY doubleValue];
      Result = YES;
    };
    
  };
      
  return Result;
};


void SetGlobalOriginOpenClip(double X,double Y,NSMutableDictionary *TheDict)
{
  NSMutableDictionary *PtDict	= SetXYPointOpenClip(X,Y);
  [TheDict setObject:PtDict forKey:OpenClip_GlobalOrigin];
};


BOOL GetInsertionPointOpenClip(NSDictionary *TheDict,NSPoint *ThePt)
{
  BOOL Result = NO;
  
  NSDictionary *PtDict = [TheDict objectForKey:OpenClip_InsertionPoint];
  
  if (PtDict) {
    NSNumber *nX = [PtDict objectForKey:OpenClip_X];
    NSNumber *nY = [PtDict objectForKey:OpenClip_Y];
    
    if (nX && nY) {
      ThePt->x = [nX doubleValue];
      ThePt->y = [nY doubleValue];
      Result = YES;
    };
    
  };  
  
  return Result;
};


void SetInsertionPointOpenClip(double X,double Y,NSMutableDictionary *TheDict)
{
  NSMutableDictionary *PtDict	= SetXYPointOpenClip(X,Y);
  [TheDict setObject:PtDict forKey:OpenClip_InsertionPoint];
};


BOOL GetLayerNameOpenClip(NSDictionary *TheNSLayer,NSString *TheLayerName)
{
  BOOL Result = NO;

  TheLayerName = [TheNSLayer objectForKey:OpenClip_LayerName];
  
  if (TheLayerName)
    Result = YES;
    
  return Result;
};



BOOL GetLayerColorOpenClip(NSDictionary *TheNSLayer,float *r,float *g,float *b,float *a)
{
  BOOL Result = NO;
  
  NSDictionary *TheColor = [TheNSLayer objectForKey:OpenClip_LayerColor];
  
  if (TheColor) {
    NSNumber *nR = [TheColor objectForKey:OpenClip_Red];
    NSNumber *nG = [TheColor objectForKey:OpenClip_Green];
    NSNumber *nB = [TheColor objectForKey:OpenClip_Blue];
    NSNumber *nA = [TheColor objectForKey:OpenClip_Alpha];
    
    if (nR && nG && nB && nA) {
      *r = [nR floatValue];
      *g = [nG floatValue];
      *b = [nB floatValue];
      *a = [nA floatValue];
      Result = YES;
    }
    
  };
  
  return Result;
};



BOOL GetLayerScaleOpenClip(NSDictionary *TheNSLayer,double *TheScale)
{
  BOOL Result = NO;
  
  NSNumber *nTheScale = [TheNSLayer objectForKey:OpenClip_LayerScale];
  
  if (nTheScale) {
    *TheScale = [nTheScale doubleValue];
    Result = YES;
  };
    
  return Result;
};



void SetObjectTypeOpenClip(NSString *TheType,NSMutableDictionary *TheObj)
{
  [TheObj setObject:TheType forKey:OpenClip_ObjectType];
};



BOOL Get2DPointOpenClip(NSArray *Points,int TheIndex,NSPoint *ThePt)
{
  BOOL Result = NO;
  
  int LastN = [Points count] - 1;
  
  if (TheIndex <= LastN) {
    NSDictionary *PtDict = [Points objectAtIndex:TheIndex];
    
    if (PtDict) {
      NSNumber *nX = [PtDict objectForKey:OpenClip_X];
      NSNumber *nY = [PtDict objectForKey:OpenClip_Y];
      
      if (nX && nY) {
        ThePt->x = [nX doubleValue];
        ThePt->y = [nY doubleValue];
        Result = YES;
      };
      
    };
  
  };
  
  return Result;
};


BOOL Get3DPointOpenClip(NSArray *Points,int TheIndex,NSPoint *ThePt,double *Z)
{
  BOOL Result = NO;
  
  int LastN = [Points count] - 1;
  
  if (TheIndex <= LastN) {
    NSDictionary *PtDict = [Points objectAtIndex:TheIndex];
    
    if (PtDict) {
      NSNumber *nX = [PtDict objectForKey:OpenClip_X];
      NSNumber *nY = [PtDict objectForKey:OpenClip_Y];
      NSNumber *nZ = [PtDict objectForKey:OpenClip_Z];
      
      if (nX && nY && nZ) {
        ThePt->x = [nX doubleValue];
        ThePt->y = [nY doubleValue];
        *Z       = [nZ doubleValue];
        Result = YES;
      };
      
    };
  
  };
  
  return Result;
};


void AddXYPointOpenClip(double X,double Y,NSMutableArray *Points)
{
  NSMutableDictionary *PtDict	= SetXYPointOpenClip(X,Y);
  [Points addObject:PtDict];
};


void AddXYZPointOpenClip(double X,double Y,double Z,NSMutableArray *Points)
{
  NSMutableDictionary *PtDict	= SetXYZPointOpenClip(X,Y,Z);
  [Points addObject:PtDict];
};


void AddPointsToObjectOpenClip(NSMutableArray *Points,NSMutableDictionary *TheObj)
{
  [TheObj setObject:Points forKey:OpenClip_Points];
};


BOOL GetObjectZOpenClip(NSDictionary *TheObj,double *Z)
{
  BOOL Result = NO;
  
  NSNumber *nZ = [TheObj objectForKey:OpenClip_Z];
  
  if (nZ) {
    *Z = [nZ doubleValue];
    Result = YES;
  };
  
  return Result;
};


void SetObjectZOpenClip(double Z,NSMutableDictionary *TheObj)
{
  NSNumber *nZ = [NSNumber numberWithDouble:Z];
  [TheObj setObject:nZ forKey:OpenClip_Z];
};


BOOL GetObjectClosedOpenClip(NSDictionary *TheObj,BOOL *Closed)
{
  BOOL Result = NO;
  
  NSNumber *nClosed = [TheObj objectForKey:OpenClip_Closed];
  
  if (nClosed) {
    *Closed = [nClosed boolValue];
    Result = YES;
  };
  
  return Result;
};


NSMutableDictionary* RGBADictionaryOpenClip(RGBColor TheColor,double Alpha)
{
  float R = (float)TheColor.red / 65535.0;
  float G = (float)TheColor.green / 65535.0;
  float B = (float)TheColor.blue / 65535.0;
  
  NSNumber *nR = [NSNumber numberWithFloat:R];
  NSNumber *nG = [NSNumber numberWithFloat:G];
  NSNumber *nB = [NSNumber numberWithFloat:B];
  NSNumber *nA = [NSNumber numberWithFloat:Alpha];
  
  NSMutableDictionary *RGBA	= [NSMutableDictionary dictionary];
  [RGBA setObject:nR forKey:OpenClip_Red];
  [RGBA setObject:nG forKey:OpenClip_Green];
  [RGBA setObject:nB forKey:OpenClip_Blue];
  [RGBA setObject:nA forKey:OpenClip_Alpha];
  
  return RGBA;
};


BOOL GetPenColorAndSetStrokeOpenClip(NSDictionary *TheObj)
{
  BOOL GotColor = NO;
  
  NSDictionary *TheColor = [TheObj objectForKey:OpenClip_PenColor];
  
  if (TheColor) {
    NSNumber *nR = [TheColor objectForKey:OpenClip_Red];
    NSNumber *nG = [TheColor objectForKey:OpenClip_Green];
    NSNumber *nB = [TheColor objectForKey:OpenClip_Blue];
    NSNumber *nA = [TheColor objectForKey:OpenClip_Alpha];
    
    if (nR && nG && nB && nA) {
      float R = [nR floatValue];
      float G = [nG floatValue];
      float B = [nB floatValue];
      float A = [nA floatValue];
      
      NSColor *PenColor = [NSColor colorWithCalibratedRed:R green:G blue:B alpha:A];
      [PenColor setStroke];
      GotColor = YES;
    }
    else
      [[NSColor blackColor] setStroke];
    
  };
  
  return GotColor;
};



void SetPenColorOpenClip(RGBColor PenColor,double PenAlpha,NSMutableDictionary *TheObj)
{
  NSMutableDictionary *TheColor = RGBADictionaryOpenClip(PenColor,PenAlpha);
  [TheObj setObject:TheColor forKey:OpenClip_PenColor];
};


BOOL GetFillColorAndSetFillOpenClip(NSDictionary *TheObj)
{
  BOOL GotColor = NO;
  
  NSDictionary *TheColor = [TheObj objectForKey:OpenClip_FillColor];
  
  if (TheColor) {
    NSNumber *nR = [TheColor objectForKey:OpenClip_Red];
    NSNumber *nG = [TheColor objectForKey:OpenClip_Green];
    NSNumber *nB = [TheColor objectForKey:OpenClip_Blue];
    NSNumber *nA = [TheColor objectForKey:OpenClip_Alpha];
    
    if (nR && nG && nB && nA) {
      float R = [nR floatValue];
      float G = [nG floatValue];
      float B = [nB floatValue];
      float A = [nA floatValue];
      
      NSColor *FillColor = [NSColor colorWithCalibratedRed:R green:G blue:B alpha:A];
      [FillColor setFill];
      GotColor = YES;
    }
    else
      [[NSColor blackColor] setFill];
    
  };
  
  return GotColor;
};


BOOL GetPenSizeOpenClip(NSDictionary *TheObj,float *TheSize)
{
  BOOL Result = NO;
  
  NSNumber *nTheSize = [TheObj objectForKey:OpenClip_PenSize];
  
  if (nTheSize) {
    *TheSize = [nTheSize floatValue];
    Result = YES;
  };
  
  return Result;
};



void GetLineDashAndSetPathDashOpenClip(NSDictionary *TheObj,NSBezierPath *ThePath)
{
  NSArray *Dashes = [TheObj objectForKey:OpenClip_LineDash];
  
  if (Dashes) {
    int TheCount = [Dashes count];
    int LastN = TheCount - 1;
        
    //float Pattern[TheCount];
    CGFloat Pattern[TheCount];
    
    for (int N = 0; N <= LastN; N++) {
      NSNumber *NbrDashLength = [Dashes objectAtIndex:N];
      double DashLength = [NbrDashLength floatValue];
      Pattern[N] = DashLength;
    };
  
    [ThePath setLineDash:Pattern count:TheCount phase:0];
  };
  
};


BOOL GetRotationAngleOpenClip(NSDictionary *TheObj,double *TheAngle)
{
  BOOL Result = NO;
  
  NSNumber *nTheAngle = [TheObj objectForKey:OpenClip_RotationAngle];
  
  if (nTheAngle) {
    *TheAngle = [nTheAngle doubleValue];
    Result = YES;
  };
  
  return Result;
};


BOOL GetStartAngleOpenClip(NSDictionary *TheObj,double *TheAngle)
{
  BOOL Result = NO;
  
  NSNumber *nTheAngle = [TheObj objectForKey:OpenClip_StartAngle];
  
  if (nTheAngle) {
    *TheAngle = [nTheAngle doubleValue];
    Result = YES;
  };
  
  return Result;
};


BOOL GetArcAngleOpenClip(NSDictionary *TheObj,double *TheAngle)
{
  BOOL Result = NO;
  
  NSNumber *nTheAngle = [TheObj objectForKey:OpenClip_ArcAngle];
  
  if (nTheAngle) {
    *TheAngle = [nTheAngle doubleValue];
    Result = YES;
  };
  
  return Result;
};


BOOL GetCornerRadiusOpenClip(NSDictionary *TheObj,double *TheRadius)
{
  BOOL Result = NO;
  
  NSNumber *nTheRadius = [TheObj objectForKey:OpenClip_CornerRadius];
  
  if (nTheRadius) {
    *TheRadius = [nTheRadius doubleValue];
    Result = YES;
  };
  
  return Result;
};



void WriteOutFileOpenClip(NSMutableDictionary *TheDocument)
{
  NSFileManager *fileManager = [NSFileManager defaultManager];
  NSArray *Paths = NSSearchPathForDirectoriesInDomains(NSApplicationSupportDirectory,NSLocalDomainMask,NO);
  
  if ([Paths count] > 0) {
    NSString *destinationPath = [[Paths objectAtIndex:0] stringByAppendingPathComponent:@"OpenClip"];
    //NSLog(destinationPath); // /Library/Application Support/OpenClip
    
    if (! [fileManager fileExistsAtPath:destinationPath]) {
      NSError* error;
      [fileManager createDirectoryAtPath:destinationPath withIntermediateDirectories:NO attributes:nil error:&error];
      //NSLog(@"Created Directory");
    };
    
    NSString *TheFilePath = [destinationPath stringByAppendingPathComponent:@"OpenClip"];
    //NSLog(TheFilePath);  // /Library/Application Support/OpenClip/OpenClip
    
    NSFileHandle *theFileHandle = [NSFileHandle fileHandleForWritingAtPath:TheFilePath];
    
    [TheDocument writeToFile:TheFilePath atomically:YES]; // writeToFile is NSData
    
    //if ([TheDocument writeToFile:TheFilePath atomically:YES])
    //  NSLog(@"WriteOutFileOpenClip Wrote file OK");
    //else
    //  NSLog(@"WriteOutFileOpenClip Did not write file");
    
    [theFileHandle closeFile];
  };
};

// same as void CopyJob::WriteToPasteboard(NSMutableDictionary *TheDocument) except for mClearPasteboard
void WriteToPasteboardOpenClip(NSMutableDictionary *TheDocument)
{
  //OSStatus err = noErr;
  
  PasteboardRef theClipboard;
  
  OSStatus err = PasteboardCreate(kPasteboardClipboard,&theClipboard);
  
  //if (mClearPasteboard)  // CopyJob has mClearPasteboard, this does not
    PasteboardClear(theClipboard);
  
  //PasteboardSyncFlags syncFlags;
  //syncFlags = PasteboardSynchronize( theClipboard ); // 2
  
  NSData *TheData = [NSKeyedArchiver archivedDataWithRootObject:TheDocument];
  //err = PasteboardPutItemFlavor(theClipboard,(PasteboardItemID)0,CFSTR("com.openclip.clip"),(CFDataRef)TheData,0);
  err = PasteboardPutItemFlavor(theClipboard,(PasteboardItemID)0,CFSTR("com.openclip.clip"),(CFDataRef)CFBridgingRetain(TheData),0);
};


//----------------------------------------------------------------------------------------------