//----------------------------------------------------------------------------------------------

// ShapeFileView.m  ShapefileToOpenClip

//----------------------------------------------------------------------------------------------

// Copyright (c) 2004-2009 Pascal Brandt - All Rights Reserved

/*
 
 
*/
//----------------------------------------------------------------------------------------------


#import "ShapeFileView.h"
#import "Shapefile.h"
#import "ShapePoint.h"
#import "NSBezierPathAdditions.h"
#import "AppController.h"

@implementation ShapeFileView

@synthesize unit = mUnit;
@synthesize zoom = mZoom;
//@synthesize printing = mPrinting;

//----------------------------------------------------------------------------------------------
/*
static void drawStringInRect(NSRect rect,NSString *str,int fontSize)
{
  //NSLog(@"ShapeFileView drawStringInRect");
  
  NSDictionary *attr = [NSDictionary dictionaryWithObject:[NSFont boldSystemFontOfSize:fontSize]
                                                   forKey:NSFontAttributeName];
	
  NSAttributedString *theStr = [[NSAttributedString alloc] initWithString:str attributes:attr];
	
  NSSize theSize = [theStr size];
	
  NSPoint pt = NSMakePoint((rect.size.width - theSize.width) / 2,
                           (rect.size.height - theSize.height) / 2);
	
  // Clear the rect
  rect.origin.x = 0;
  rect.origin.y = 0;
  [[NSColor whiteColor] set];
  NSRectFill(rect);

  [theStr drawAtPoint:pt];
}
*/
//----------------------------------------------------------------------------------------------


- (void)awakeFromNib
{
  //NSLog(@"ShapeFileView awakeFromNib");
  [[self window] makeFirstResponder:self];
  [[self window] setAcceptsMouseMovedEvents:YES];
  
  NSArray *dragShapefile = [NSArray arrayWithObject:NSFilenamesPboardType];
  [self registerForDraggedTypes:dragShapefile];
  
  mList = [[ShapefileList alloc] init];
  
  mUnit = kMeters;
  //[self setZoom:1.0];
  mZoom = 1.0;
  //mPrinting = false;
  
  //[nbrObjectsText setStringValue:@""];
  
  [self setNeedsDisplay:YES];
}


- (void)reset
{
  [mList reset];
  
  //[self setZoom:1.0];
  mZoom = 1.0;
  //[nbrObjectsText setStringValue:@""];
  
};


- (BOOL)isOpaque
{
  return YES;
}


- (void)addShapefile:(Shapefile*)thefile
{
  [mList addShapefile:thefile];
}


- (long)nbrShapefiles
{
  return [mList nbrShapefiles];
};



- (void)print:(id)sender
{
  //NSLog(@"OpenClipView print");
  [[NSPrintOperation printOperationWithView:self] runOperation];
}

// all this is from MPGChartView in Benchmark.  I think it's incomplete

- (void)updateFrame
{
  if ([self superview]) {
    //NSRect r = NSZeroRect;

    //r.size = [mPrintInfo paperSize];
    //[self setFrame:r];
  }
}


- (void)setPrintInfo:(NSPrintInfo *)p
{
  if (mPrintInfo != p) {
    //[mPrintInfo autorelease];
    //mPrintInfo = [p copyWithZone:[self zone]];
    [self updateFrame];
    [self setNeedsDisplay:YES];
  }
}


- (NSPrintInfo *)printInfo
{
  return mPrintInfo;
}


- (void)updateChartSize:(NSNotification *)n
{
  [self setNeedsDisplay:YES];
}


- (void)copyShapefiles:(SHPunit)theUnit
{
  long NbrShapeFiles = [mList nbrShapefiles];
  
  if (NbrShapeFiles > 0) {
    //NSLog(@"ShapeFileView copyShapefiles");
    
    NSRect BBox = mList.boundingBox;
    double PageWidth = 36.0 * 72.0;
    double ScaleX = NSWidth(BBox) / PageWidth;
    
    //NSLog(@"copyShapefiles ScaleX: %.4f",ScaleX);  // 1.0610  58.6127
    
    ScaleX = ((long)(ScaleX * 100)) / 100.0;
    
    //NSLog(@"copyShapefiles ScaleX: %.4f",ScaleX);  // 1.06    58.61
    
    double ScaleFactor = 1.0 / ScaleX;
    
    //NSLog(@"copyShapefiles ScaleFactor: %.4f",ScaleFactor);
    
    NSAffineTransform *t = [NSAffineTransform transform];
    
    [t scaleBy:ScaleFactor];
    
    double deltaX = -NSMinX(BBox);
    double deltaY = -NSMinY(BBox);
    
    [t translateXBy:deltaX yBy:deltaY];
    
    [mList openClipCopy:t theScale:ScaleX];
  };
  
};


- (void)drawShapefiles:(NSRect)rectBounds
{
  long NbrShapeFiles = [mList nbrShapefiles];
  
  if (NbrShapeFiles > 0) {
    
    NSAffineTransform *t = [NSAffineTransform transform];
    NSRect BBox = mList.boundingBox;
    //NSLog(@"drawRect BBox %@",NSStringFromRect(BBox));
    
    double ScaleX = NSWidth(BBox) / NSWidth(rectBounds);
    double ScaleY = NSHeight(BBox) / NSHeight(rectBounds);
    
    //NSLog(@"ScaleX: %.2f ScaleY: %.2f",ScaleX,ScaleY);  // 3.99 2.90
    
    double ScaleFactor1 = 1.0 / fmax(ScaleX,ScaleY);
    
    [t scaleBy:ScaleFactor1];
    
    double deltaX = -NSMinX(BBox);
    double deltaY = -NSMinY(BBox);
    
    //if ([NSGraphicsContext currentContextDrawingToScreen]) {
      double Difference1 = fabs(ScaleY - ScaleX);
      
      if (ScaleX < ScaleY)
        deltaX += ((NSWidth(rectBounds) * Difference1) / 2.0);
      else if (ScaleX > ScaleY)
        deltaY += ((NSHeight(rectBounds) * Difference1) / 2.0);
      
    //}
    //else {
      // center this better
    //};
    
    [t translateXBy:deltaX yBy:deltaY];
    
    //mList.lineWidth = (1 * ScaleFactor1);
    [mList draw:t];
  }
};


- (void)drawRect:(NSRect)rect
{
  NSRect rectBounds = [self bounds];
  
  if ([NSGraphicsContext currentContextDrawingToScreen]) {
    // draw screen-only elements here
    [[NSColor whiteColor] set];
    NSRectFill(rectBounds);
  }
  else {
    // draw printer-only elements here
  };

  //if ([self inLiveResize]) {
  //  NSString *str = [NSString stringWithFormat:@"Resizing to %g x %g",frame.size.width,frame.size.height];
  //  drawStringInRect(frame, str, 20);
  //
  //  return;
  //}

  [self drawShapefiles:rectBounds];

	
}

/*
- (NSData*)PDFForView
{
  NSRect boundsRect = [self bounds];
  //NSRect frameRect = [self frame];
  //NSLog(@"ShafeFileView bounds %@",NSStringFromRect(boundsRect));  // {{0, 0}, {690, 583}}
  //NSLog(@"ShafeFileView frame %@",NSStringFromRect(frameRect));    // {{21, 92}, {690, 583}}
	NSData *dataToSave = [self dataWithPDFInsideRect:boundsRect];
  //NSData *dataToSave = [self dataWithPDFInsideRect:frameRect];
	
	return dataToSave;
};
*/
/*
- (NSData*)TIFFForView
{
  NSImage *image = [[NSImage alloc] initWithData:[self PDFForView]];
  
  return [image TIFFRepresentation];
};
*/

/*
- (void)displayZoomFactor
{
  NSString* strZoom = [NSString stringWithFormat:@"%ld%%",(long)(lround(mZoom * 100))];
  [zoomFactorText setStringValue:strZoom];
}
*/

/*
- (void)centerView
{
  //NSLog(@"ShapeFileView centerView");
  
  NSRect FrameRect   = [self frame];
  double FrameWidth  = NSWidth(FrameRect);
  double FrameHeight = NSHeight(FrameRect);
  
  double FrameWidthM  = FrameWidth * METERS_PER_PIXEL;
  double FrameHeightM = FrameHeight * METERS_PER_PIXEL;
  double AdjFrameWidthM  = FrameWidth * (METERS_PER_PIXEL / 2);
  double AdjFrameHeightM = FrameHeight * (METERS_PER_PIXEL / 2);
  
  NSRect BBox = mList.boundingBox;
  
  double OriginX = NSMidX(BBox) - AdjFrameWidthM;
  double OriginY = NSMidY(BBox) - AdjFrameHeightM;
  
  mViewRect = NSMakeRect(OriginX,OriginY,FrameWidthM,FrameHeightM);
  
  [self displayViewRect];
  [self displayFrameRect];
};
*/

- (void)zoomViewIn:(NSEvent*)theEvent
{
  //NSLog(@"ShapeFileView zoomViewIn");
  
  //NSPoint WindowLoc = [theEvent locationInWindow];
  //NSPoint MousePt = [self convertPoint:WindowLoc fromView:nil];
  //NSPoint MousePtM = [self pt2Meters:MousePt];
  
  //float nZoom = mZoom;
  
  //nZoom = (nZoom * sqrt(2));
  
  //mZoom = nZoom;
  //[self setZoomUnitFactor];
  //[self setZoom:nZoom];
  //[self setZoom:(mZoom * sqrt(2))];
  mZoom *= sqrt(2);
  //[self displayZoomFactor];
  
  //double WidthM = NSWidth(mViewRect);
  //double HeightM = NSHeight(mViewRect);
  
  //double ZoomedWidthM  = WidthM / sqrt(2);
  //double ZoomedHeightM = HeightM / sqrt(2);
  //NSPoint AboutPt = MousePtM;
  
  //double OriginX = AboutPt.x - (ZoomedWidthM / 2);
  //double OriginY = AboutPt.y - (ZoomedHeightM / 2);
  
  //mViewRect = NSMakeRect(OriginX,OriginY,ZoomedWidthM,ZoomedHeightM);
  
  //[self displayViewRect];
  //[self setNeedsDisplay:YES];
}


- (void)zoomViewOut:(NSEvent*)theEvent
{
  //NSLog(@"ShapeFileView zoomViewOut");
  
  //float nZoom = mZoom;
  
  //nZoom = (nZoom / sqrt(2));
  
  //mZoom = nZoom;
  //[self setZoomUnitFactor];
  //[self setZoom:(mZoom / sqrt(2))];
  mZoom /= sqrt(2);
  //[self displayZoomFactor];
  
  //double WidthM = NSWidth(mViewRect);
  //double HeightM = NSHeight(mViewRect);
  
  //double ZoomedWidthM  = WidthM * sqrt(2);
  //double ZoomedHeightM = HeightM * sqrt(2);
  //NSPoint AboutPt = NSMakePoint(NSMidX(mViewRect),NSMidY(mViewRect));
  
  //double OriginX = AboutPt.x - (ZoomedWidthM / 2);
  //double OriginY = AboutPt.y - (ZoomedHeightM / 2);
  
  //mViewRect = NSMakeRect(OriginX,OriginY,ZoomedWidthM,ZoomedHeightM);
  
  //[self displayViewRect];
  //[self setNeedsDisplay:YES];
}


- (void)scrollWheel:(NSEvent*)theEvent
{
  //NSLog(@"ShapeFileView scrollWheel");
  
  //float DeltaX = [theEvent deltaX];  // left is negative, right is positive
  float DeltaY = [theEvent deltaY];  // up is negative, down is positive
  
  bool RollForward = (DeltaY > 0.0);
  bool RollBack    = (DeltaY < 0.0);
  
  if (RollForward)
    [self zoomViewOut:theEvent];
  else if (RollBack)
    [self zoomViewIn:theEvent];
  
};


- (void)mouseDown:(NSEvent *)theEvent
{
  //NSLog(@"ShapeFileView mouseDown"); // called

  NSPoint WindowLoc = [theEvent locationInWindow];
  NSPoint MousePt = [self convertPoint:WindowLoc fromView:nil];

  mStartPoint = MousePt;
}


- (void)mouseDragged:(NSEvent *)theEvent
{
  //NSLog(@"ShapeFileView mouseDragged"); // called

  //NSPoint StartPtM = [self pt2Meters:mStartPoint];

  //NSPoint WindowLoc = [theEvent locationInWindow];
  //NSPoint MousePt = [self convertPoint:WindowLoc fromView:nil];
  //NSPoint MousePtM = [self pt2Meters:MousePt];
  //mStartPoint = MousePt;

  //NSPoint DeltaM = NSMakePoint(MousePtM.x - StartPtM.x,MousePtM.y - StartPtM.y);
  
  //mViewRect.origin.x = mViewRect.origin.x - DeltaM.x;
  //mViewRect.origin.y = mViewRect.origin.y - DeltaM.y;

  [self setNeedsDisplay:YES];

}


- (void)mouseMoved:(NSEvent *)theEvent
{
  //NSLog(@"ShapeFileView mouseMoved");  // called
  long NbrShapeFiles = [mList nbrShapefiles];
  
  if (NbrShapeFiles > 0) {

    NSPoint WindowLocP = [theEvent locationInWindow];
    NSPoint MousePt = [self convertPoint:WindowLocP fromView:nil];

    BOOL MouseInside = [self mouse:MousePt inRect:[self bounds]];

    if(MouseInside) {
      //NSPoint MousePtM = [self pt2Meters:MousePt];

      //NSString* aString = [NSString stringWithFormat:@"East: %ld North: %ld",(long)MousePtM.x,(long)MousePtM.y];

      //[mShowLocText setStringValue:aString];
    }

  }

}



- (void)mouseUp:(NSEvent *)theEvent
{
  //NSLog(@"ShapeFileView mouseUp"); // called
}


- (void)viewWillStartLiveResize
{
  //NSLog(@"ShapeFileView viewWillStartLiveResize");
  
  [super viewWillStartLiveResize];
  [self setNeedsDisplay:NO];
  // Could do something here if needed
}


- (void)viewDidEndLiveResize
{
  //NSLog(@"ShapeFileView viewDidEndLiveResize");
  
  [super viewDidEndLiveResize];
  
  /*
  NSRect FrameRect = [self frame];

  NSRect ViewRectM = mViewRect;

  long nWidth = NSWidth(FrameRect) * mZoomUnitFactor;
  long nHeight = NSHeight(FrameRect) * mZoomUnitFactor;
  ViewRectM.size = NSMakeSize(nWidth,nHeight);

  mViewRect = ViewRectM;
  */
  
  [self setNeedsDisplay:YES];
  // Could do something here if needed
}


- (unsigned int)draggingEntered:(id <NSDraggingInfo>)sender
{
  //NSLog(@"ShapeFileView draggingEntered"); // called
  
  return NSDragOperationMove;
}


- (unsigned int)draggingUpdated:(id <NSDraggingInfo>)sender
{
  //NSLog(@"ShapeFileView draggingUpdated"); // called
  
  //NSPasteboard *pboard = [sender draggingPasteboard];

  return NSDragOperationMove;
}


- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
  //NSLog(@"ShapeFileView performDragOperation");  // called

  NSPasteboard *pboard = [sender draggingPasteboard];

  if ([[pboard types] containsObject:NSFilenamesPboardType]) {
    [self reset];
    
    NSArray *files = [pboard propertyListForType:NSFilenamesPboardType];
    long numberOfFiles = [files count];

    for(long I = 0; I < numberOfFiles; I++) {
      //NSLog(@"ShapeFileView open one shape file");
      
      NSString *thePath = [files objectAtIndex:I];
      //NSLog(@"performDragOperation %ld strShapefile %@",I,thePath);
      
      Shapefile *theShapefile = [[Shapefile alloc] initWithPath:thePath];
      
      if (theShapefile)
        [self addShapefile:theShapefile];
      
    }
    
    //[self centerView];
    //[self displayNbrShapefiles];
    [self setNeedsDisplay:YES];
    // Perform operation using the list of files
  }

  return YES;
}


- (BOOL)acceptsFirstResponder
{
  //NSLog(@"Accepting");
  return YES;
}


- (BOOL)resignFirstResponder
{
  //NSLog(@"Resigning");
  return YES;
}


- (BOOL)becomeFirstResponder
{
  //NSLog(@"Becoming");
  return YES;
}


@end