//----------------------------------------------------------------------------------------------

// Shapefile.m  ShapefileToOpenClip

//----------------------------------------------------------------------------------------------

// Copyright (c) 2004-2009 Pascal Brandt - All Rights Reserved

/*
 
 
*/
//----------------------------------------------------------------------------------------------


#import "Shapefile.h"
#import "ShapePolyline.h"
#import "ShapePoint.h"
#import "NSBezierPathAdditions.h"
#import "OpenClipDefines.h"
#import "OpenClipUtils.h"

@implementation Shapefile

@synthesize bbox = mBBox;

@synthesize fileLength = mFileLength;
@synthesize	recordCount = mRecordCount;

@synthesize shapefileType = mShapefileType;

//@synthesize lineWidth = mLineWidth;


// -------------------------------
// C endian routines

int convertToLittleEndianInteger(void* pVal)
{
  int dwResult;

  memcpy((void*) ((unsigned long) &dwResult), (void*) ((unsigned long) pVal + 3), 1);
  memcpy((void*) ((unsigned long) &dwResult + 1), (void*) ((unsigned long) pVal + 2), 1);
  memcpy((void*) ((unsigned long) &dwResult + 2), (void*) ((unsigned long) pVal + 1), 1);
  memcpy((void*) ((unsigned long) &dwResult + 3), (void*) ((unsigned long) pVal), 1);

  return dwResult;
}


long convertToLittleEndianLong(long Val)
{
  long dwResult = Val;

  memcpy((void*) ((unsigned long) &dwResult), (void*) ((unsigned long) &Val + 3), 1);
  memcpy((void*) ((unsigned long) &dwResult + 1), (void*) ((unsigned long) &Val + 2), 1);
  memcpy((void*) ((unsigned long) &dwResult + 2), (void*) ((unsigned long) &Val + 1), 1);
  memcpy((void*) ((unsigned long) &dwResult + 3), (void*) ((unsigned long) &Val), 1);

  return dwResult;
}

// -------------------------------

- (id)initWithPath:(NSString*)theFilePath
{
  self = [super init];
  
  if (self) {
    //mBBox
    
    //mFileLength
    //mVersion
    //mRecordCount
    //mShapefileType
    //mLineWidth = 1;
    
    if (![self loadOneShapefile:theFilePath])
      self = nil;
    
  }
  
  return self;
};


- (NSString *)shapefileTypeAsString
{
  NSString* strShapefileType;
	
  switch(self.shapefileType) {
    case 0:  strShapefileType = @"Null Shape";
             break;
    case 1:  strShapefileType = @"Point";
             break;
    case 3:  strShapefileType = @"PolyLine";
             break;
    case 5:  strShapefileType = @"Polygon";
             break;
    case 8:  strShapefileType = @"MultiPoint";
             break;
    case 11: strShapefileType = @"PointZ";
             break;
    case 13: strShapefileType = @"PolyLineZ";
             break;
    case 15: strShapefileType = @"PolygonZ";
             break;
    case 18: strShapefileType = @"MultiPointZ";
             break;
    case 21: strShapefileType = @"PointM";
             break;
    case 23: strShapefileType = @"PolyLineM";
             break;
    case 25: strShapefileType = @"PolygonM";
             break;
    case 28: strShapefileType = @"MultiPointM";
             break;
    case 31: strShapefileType = @"MultiPatch";
             break;
    default: strShapefileType = @"unknown";
             break;
  }
	
  return strShapefileType;
}


- (BOOL)badShapeType
{
  BOOL Result = YES;
  
  switch(self.shapefileType) {
    case 1: Result = NO;  // Point
            break;
    case 3: Result = NO;  // PolyLine
            break;
    case 5: Result = NO;  // Polygon
            break;
  }

  return Result;
};


- (void*)parsePoint:(void *)pMain
{
  double TheEast;
  double TheNorth;
  
  memcpy(&TheEast,  (void*) ((unsigned long) pMain), 8);
  memcpy(&TheNorth, (void*) ((unsigned long) pMain + 8), 8);
	
  ShapePoint *ThePt = [[ShapePoint alloc] initWithEast:TheEast North:TheNorth];
  
  [mObjList addObject:ThePt];
	
  pMain = (void*) ((unsigned long) pMain + 16);
	
  return pMain;
}


- (void*)parsePolyline:(void *)pMain
{
  ShapePolyline *ThePoly = [[ShapePolyline alloc] init];
	
  double  BoundingBox[4];
  for(long I = 0; I <= 3; I++) {
    memcpy(&(BoundingBox[I]), pMain, 8);
    pMain = (void*) ((unsigned long) pMain + 8);
  }
	
  ThePoly.bbox = NSMakeRect(BoundingBox[0],BoundingBox[1],BoundingBox[2],BoundingBox[3]);
  
  SInt32 NbrParts;
  memcpy(&NbrParts, pMain, 4);
  ThePoly.numParts = NbrParts;
  pMain = (void*) ((unsigned long) pMain + 4);
  
  SInt32 NbrPoints;
  memcpy(&NbrPoints, pMain, 4);
  ThePoly.numPoints = NbrPoints;
  pMain = (void*) ((unsigned long) pMain + 4);
	
  for(long I = 0; I < NbrParts; I++) {
    SInt32 ThePart;
    memcpy(&ThePart, (void*) ((unsigned long) pMain + 4 * I), 4);
    NSNumber *Partn = [[NSNumber alloc] initWithInt:ThePart];
    [ThePoly->mParts addObject:Partn];
  }
	
  double TheEast;
  double TheNorth;
  
  // read the elements
  for(long I = 0; I < NbrPoints; I++) {
    memcpy(&TheEast, (void*) ((unsigned long) pMain + 16 * I + (NbrParts * 4) ), 8);
    memcpy(&TheNorth, (void*) ((unsigned long) pMain + 16 * I + (NbrParts * 4) + 8), 8);
    
    ShapePoint *ThePt = [[ShapePoint alloc] initWithEast:TheEast North:TheNorth];
		
    if (ThePt)
      [ThePoly->mPoints addObject:ThePt];
  }
	
  if (ThePoly.numPoints > 0)
    [mObjList addObject:ThePoly];
  else
    NSLog(@"parsePolyline no points in ShapePolyLine");
	
  pMain = (void*) ((unsigned long) pMain + (4 * NbrParts) + (16 * NbrPoints));
	
  return pMain;
}


- (BOOL)loadOneShapefile:(NSString*)theFilePath;
{
  mObjList = [[NSMutableArray alloc] init];

  NSData* TheData = [NSData dataWithContentsOfFile:theFilePath];

  //char *pBufferShapefile = malloc([TheData length]);
  char *pBufferShapefile = (char*)malloc([TheData length]);
  [TheData getBytes:pBufferShapefile];

  void *pMain = &pBufferShapefile[0];

  // magic number of header block does not match (9994)
  if(convertToLittleEndianInteger(pMain) != 0x270a) {
    return NO;
  }
	
  // go to file length
  pMain = (void*) ((unsigned long) pMain + 24);	
  mFileLength = 2 * convertToLittleEndianInteger(pMain);

  // go to version number
  pMain = (void*) ((unsigned long) pMain + 4);
  memcpy(&mVersion, pMain, 4);

  // version number should match (1000)
  if(mVersion != 0x03e8) {
    return NO;
  }
	
  // go to shape type
  pMain = (void*) ((unsigned long) pMain + 4);
  SInt32     nShapefileType;
  memcpy(&nShapefileType, pMain, 4);
  self.shapefileType = nShapefileType;
  
  if([self badShapeType]) {
    //NSRunAlertPanel(@"ShpViewer", @"Shapetype %d not yet supported!", @"OK", nil, nil, (int)nShapefileType);
    NSLog(@"ShpViewer Shapetype %d not yet supported!",(int)nShapefileType);
    return NO;
  }

  pMain = (void*) ((unsigned long) pMain + 4);

  // get bounding box
  double dExtendLeft;
  memcpy(&dExtendLeft, pMain, 8);
  pMain = (void*) ((unsigned long) pMain + 8);

  double dExtendBottom;
  memcpy(&dExtendBottom, pMain, 8);
  pMain = (void*) ((unsigned long) pMain + 8);

  double dExtendRight;
  memcpy(&dExtendRight, pMain, 8);
  pMain = (void*) ((unsigned long) pMain + 8);

  double dExtendTop;
  memcpy(&dExtendTop, pMain, 8);
  pMain = (void*) ((unsigned long) pMain + 40);

  self.bbox = NSMakeRect(dExtendLeft,dExtendBottom,dExtendRight-dExtendLeft,dExtendTop-dExtendBottom);

  SInt32 TotalContentLength = 100;
  
  while(TotalContentLength <= mFileLength) {
    SInt32     nRecord;
    memcpy(&nRecord, pMain, 4);
    mRecordCount = (SInt32)convertToLittleEndianLong(nRecord);
    pMain = (void*) ((unsigned long) pMain + 4);

    SInt32	   nContentLength = 0;
    memcpy(&nContentLength, pMain, 4);
    nContentLength = (SInt32)convertToLittleEndianLong(nContentLength);

    TotalContentLength = TotalContentLength + (2 * nContentLength) + 8;

    pMain = (void*) ((unsigned long) pMain + 4);
    memcpy(&nShapefileType, pMain, 4);
    pMain = (void*) ((unsigned long) pMain + 4);

    if(nShapefileType == 1) // Point
      pMain = [self parsePoint:pMain];

    if((nShapefileType == 3) || (nShapefileType == 5)) // PolyLine Polygon
      pMain = [self parsePolyline:pMain];

    if(TotalContentLength == mFileLength) {
      return YES;
    }

  }

  return YES;
}

- (void)drawShapePoint:(NSAffineTransform*)t
{
  //NSLog(@"ShapeFileView drawShapePoint with file");
  
  long NbrShapes = [mObjList count];
  
  [[NSColor redColor] set];

  for(long I = 0; I < NbrShapes; I++) {
    ShapePoint* ThePt = [mObjList objectAtIndex:I];

    //NSPoint ptToDraw = NSMakePoint(ThePt.east,ThePt.north);
    NSPoint ptToDraw = [t transformPoint:NSMakePoint(ThePt.east,ThePt.north)];
    
    NSBezierPath *ptPath = [NSBezierPath bezierPath];
    [ptPath moveToPoint:ptToDraw];
    [ptPath appendBezierPathWithCrossOfSize:4];
    [ptPath stroke];
  }

}


- (void)copyShapePoint:(NSAffineTransform*)t theList:(NSMutableArray *)TheList
{
  //NSLog(@"ShapeFileView drawShapePoint with file");
  
  long NbrShapes = [mObjList count];
  
  //[[NSColor redColor] set];

  for(long I = 0; I < NbrShapes; I++) {
    ShapePoint* ThePt = [mObjList objectAtIndex:I];

    //NSPoint ptToDraw = NSMakePoint(ThePt.east,ThePt.north);
    NSPoint ptToDraw = [t transformPoint:NSMakePoint(ThePt.east,ThePt.north)];
    
    NSMutableDictionary *ThisObj = [NSMutableDictionary dictionary];
    SetObjectTypeOpenClip(OpenClip_PointObj,ThisObj);
  
    //RGBColor ThePenColor = *TheObj.GetPenColor();
    //double PenAlpha = TheObj.GetPenAlpha();
   // SetPenColor(ThePenColor,PenAlpha,ThisObj);
    
   // HandleZHeight(TheObj,ThisObj);
    
    //CDPoint A = TheObj.DataPoint();
   // A = CorrectPoint(A);
    
    NSMutableArray *Points = [NSMutableArray array];
    AddXYPointOpenClip(ptToDraw.x,ptToDraw.y,Points);
    AddPointsToObjectOpenClip(Points,ThisObj);
    
    [TheList addObject:ThisObj];
  }


};


- (void)drawShapePolyline:(NSAffineTransform*)t
{
  //NSLog(@"ShapeFileView drawShapePolyline");
  //static long TempNbr = 1;
  
  [[NSColor blackColor] set];
  
  long NbrShapes = [mObjList count];

  for(long I = 0; I < NbrShapes; I++) {
    ShapePolyline* ThePoly = [mObjList objectAtIndex:I];
    
    long NbrParts = [ThePoly->mParts count];
    
    for(long J = 0; J < NbrParts; J++) {
      long NextJ = J + 1;
      NSBezierPath *TheLinePath = [NSBezierPath bezierPath];

      long NbrPoints = [ThePoly->mPoints count];

      NSNumber* startPartn = [ThePoly->mParts objectAtIndex:J];
      long StartPart = [startPartn intValue];
      long EndPart;

      if (NextJ == NbrParts) {
        EndPart = NbrPoints;
      }
      else {
        NSNumber* endPartn = [ThePoly->mParts objectAtIndex:NextJ];
        EndPart = [endPartn intValue];
      }
      
      //NSLog(@"StartPart: %ld EndPart: %ld",StartPart,EndPart);
      
      for(long K = StartPart; K < EndPart; K++) {
        ShapePoint* ThePt = [ThePoly->mPoints objectAtIndex:K];
        
        NSPoint ptToDraw = [t transformPoint:NSMakePoint(ThePt.east,ThePt.north)];
        
        if(K == StartPart)
          [TheLinePath moveToPoint:ptToDraw];
        else
          [TheLinePath lineToPoint:ptToDraw];

      }

      [TheLinePath setLineWidth:0.5];
      //[TheLinePath setLineWidth:(mLineWidth)];
      [TheLinePath stroke];
    }

  }

};


- (void)copyShapePolyline:(NSAffineTransform*)t theList:(NSMutableArray *)TheList
{
  //NSLog(@"ShapeFileView drawShapePolyline");
  
  long NbrShapes = [mObjList count];

  for(long I = 0; I < NbrShapes; I++) {
    ShapePolyline* ThePoly = [mObjList objectAtIndex:I];
    
    long NbrParts = [ThePoly->mParts count];
    
    for(long J = 0; J < NbrParts; J++) {
      long NextJ = J + 1;
      //NSBezierPath *TheLinePath = [NSBezierPath bezierPath];

      long NbrPoints = [ThePoly->mPoints count];

      NSNumber* startPartn = [ThePoly->mParts objectAtIndex:J];
      long StartPart = [startPartn intValue];
      long EndPart;

      if (NextJ == NbrParts) {
        EndPart = NbrPoints;
      }
      else {
        NSNumber* endPartn = [ThePoly->mParts objectAtIndex:NextJ];
        EndPart = [endPartn intValue];
      }
      
      //NSLog(@"StartPart: %ld EndPart: %ld",StartPart,EndPart);
      
      NSMutableDictionary *ThisObj = [NSMutableDictionary dictionary];
      SetObjectTypeOpenClip(OpenClip_PolygonObj,ThisObj);
      
      //[NSColor blackColor];
      //RGBColor ThePenColor = *TheObj.GetPenColor();
      //double PenAlpha = TheObj.GetPenAlpha();
      //SetPenColor(ThePenColor,PenAlpha,ThisObj);
      
      //float ThePenSize = TheObj.GetPenSize();
      //SetPenSize(ThePenSize,ThisObj);
      
      //SetLineDash(TheObj,ThisObj);
      
      //HandleZHeight(TheObj,ThisObj);
      //SetFillColor(TheObj,ThisObj);
      //SetClosed(TheObj.Closed(),ThisObj);
      
      NSMutableArray *Points = [NSMutableArray array];
      
      for(long K = StartPart; K < EndPart; K++) {
        ShapePoint* ThePt = [ThePoly->mPoints objectAtIndex:K];
        
        NSPoint ptToDraw = [t transformPoint:NSMakePoint(ThePt.east,ThePt.north)];
        
        AddXYPointOpenClip(ptToDraw.x,ptToDraw.y,Points);
      }
      
      AddPointsToObjectOpenClip(Points,ThisObj);
      
      [TheList addObject:ThisObj];
    }

  }

};


@end