//----------------------------------------------------------------------------------------------

// ShapefileList.m  ShapefileToOpenClip

//----------------------------------------------------------------------------------------------

/*
 

*/
//----------------------------------------------------------------------------------------------

#import "ShapefileList.h"
#import "OpenClipDefines.h"
#import "OpenClipUtils.h"

@implementation ShapefileList

@synthesize boundingBox = mBoundingBox;
//@synthesize lineWidth = mLineWidth;

- (id)init
{
  self = [super init];
  
  if (self) {
    mBoundingBox = NSMakeRect(0,0,0,0);
    //mLineWidth = 1;
    mObjs = [[NSMutableArray alloc] init];
  }
  
  return self;
}


- (void)reset
{
  mBoundingBox = NSMakeRect(0,0,0,0);
  [mObjs removeAllObjects];
};


- (void)addShapefile:(Shapefile*)theFile
{
  if ([mObjs count] == 0) {
    mBoundingBox = theFile.bbox;
    //NSLog(@"addShapefile: %d",0);
  }
  else
    mBoundingBox = NSUnionRect(mBoundingBox,theFile.bbox);
  
  [mObjs addObject:theFile];
};


- (long)nbrShapefiles
{
  return [mObjs count];
};


- (Shapefile*)shapeFileAtIndex:(long)N
{
  return [mObjs objectAtIndex:N];
};


- (void)draw:(NSAffineTransform*)t
{

  long LastN = [self nbrShapefiles] - 1;
  
  for (long N = 0; N <= LastN; N++) {
    Shapefile* TheFile = [mObjs objectAtIndex:N];
    
    switch (TheFile.shapefileType) {
      case 1: [TheFile drawShapePoint:t];
              break;
      case 3:
      case 5: [TheFile drawShapePolyline:t];
              break;
    };
    
  };
};




- (void)openClipCopy:(NSAffineTransform*)t theScale:(double)theScale
{
  NSLog(@"ShapefileList openClipCopy");
  
  //NSAutoreleasePool *thePool = [[NSAutoreleasePool alloc] init];
  //WtStAutoreleasePool savedPool;
  
  NSMutableDictionary *TheDocument = [NSMutableDictionary dictionary];
  
  [TheDocument setObject:OpenClip_WildToolsInfo forKey:OpenClip_Creator];
  
  SetScaleOpenClip(theScale,TheDocument);
  
  SetGlobalOriginOpenClip(12345.0,54321.0,TheDocument);
  // The idea of a global origin is for very large drawings where the drawing is in a
  // state plane coordinate system.  This feature is provided in OpenClip with the tags
  // needed, but we've never actually used it.
  
  SetInsertionPointOpenClip(100.0,200.0,TheDocument);
  // The idea behind an insertion point is that some programs have a 'paste at mouse'
  // capability, and the tags are provided for that purpose, but we've never actually
  // used it.
  
  // write out flat file
  
  NSMutableArray *TheNSList	= [NSMutableArray array];
  
  long LastN = [self nbrShapefiles] - 1;
  
  for (long N = 0; N <= LastN; N++) {
    Shapefile* TheFile = [mObjs objectAtIndex:N];
    
    switch (TheFile.shapefileType) {
      case 1: [TheFile copyShapePoint:t theList:TheNSList];
              break;
      case 3:
      case 5: [TheFile copyShapePolyline:t theList:TheNSList];
              break;
    };
    
  };
  
  if ([TheNSList count] > 0) {
    [TheDocument setObject:TheNSList forKey:OpenClip_DrawingObjects];
    WriteOutFileOpenClip(TheDocument);
    //long			 Ticks2 = TickCount();
    WriteToPasteboardOpenClip(TheDocument);
  };
  
  //[thePool drain];
};


@end